
        list    p=16C84
        constant RESETVECT = 0x0000
        constant PROGMEM = 0x0005

;*******************************************************************
;                             ESTOP.ASM
;              EMERGENCY STOP PROGRAM FOR MOBILE ROBOT
;*******************************************************************
;
;       INPUTS AND OUTPUTS
;       DIRECTION       PORT    PIN     DESIGNATOR
;       ---------------------------------------------------
;       inputs          portb    0      INPULSE (pulse input)
;                       portb    1      LESTOP (local EStop button)
;                       portb    2      LGO (local GO button)   
;
;       outputs         portb    3      ESTOP1  (emergency stop)
;                       portb    4      GO (procede)
;                       portb    5      LC (lost carrier)
;                       portb    6      ESTOP2 (emergency stop 2)
;
;*******************************************************************

INDIRECT equ    0x00
RTCC    equ     0x01
PC      equ     0x02
STATUS  equ     0x03
FSR     equ     0x04
PORTA   equ     0x05
PORTB   equ     0x06
PORTC   equ     0x07
flags   equ     0x0D            ; Misc flags
GoCount equ     0x0E            ; General purpose temporary register
EstpCnt equ     0x0F
delay1  equ     0x10            ; Delay between tests of input
delay2  equ     0x11            ; Delay between frames
msCount equ     0x12
LCHits  equ     0x13            ; add # hits when LC


CARRY   equ     0               ; Status Register bits
DCARRY  equ     1
ZFLAG   equ     2
PDFLAG  equ     3
TOFLAG  equ     4
PA0     equ     5
PA1     equ     6
PA2     equ     7

W       equ     0               ; W register as destination
SAME    equ     1
LSB     equ     0


                        ; flag register bits
BPSF    equ     0               ; bad pulse short flag - to short
GOF     equ     1               ; right button flag - short pulse = go
NORMF   equ     2               ; normal pulse flag - no butons pressed = do nothing
ESTOPF  equ     3               ; left button flag - long pulse = emergency stop
BPLF    equ     4               ; bad pulse long flag - to long
BFF     equ     5               ; bad frame flag - not recieving pulse start
LESTOPF equ     6               ; local ESTOP flag - local ESTOP button press
LGOF    equ     7               ; local GO flag - local GO button press


                        ; portb pin assignments
INPULSE equ     0               ; pulse input
LESTOP  equ     1               ; local ESTOP button input
LGO     equ     2               ; local GO button input
ESTOP1  equ     3               ; emergency stop output
GO      equ     4               ; procede output
LC      equ     5               ; lost carrier output
ESTOP2  equ     6               ; emergency stop output

PORTBIO equ     (1<<ESTOP1 | 1<<ESTOP2 | 1<<GO | 1<<LC) ^ 0xFF  ; tris

DefaultDelay1   equ     d'3'    ; delay about 0.01 mS
DefaultDelay2   equ     d'22'   ; delay about 0.08 mS
Debounce        equ     d'25'   ; delay about 1.5 mS

        ;signal region widths: thresholds*0.01 mS
BPSTHD  equ     d'80'
GOTHD   equ     d'120'
NORMTHD equ     d'170'
ESTPTHD equ     d'210'
   ; any pulse longer than ESTPTHD is in BPL region

NumHits equ     d'10'           ; number of hits in each region before action
LCAddHits       equ     d'20'   ; # hits to add when LC

        ; signal frame thresholds*0.08mS
SFRAME  equ     d'152'          ; short frame limit = 12 ms
LFRAME  equ     d'157'          ; long frame limit = 14 ms






        org     RESETVECT
        goto    start

        org     PROGMEM


start   clrf    FSR
        clrf    flags
        clrf    msCount

        clrf    PORTB           ; setup IO
        movlw   PORTBIO         
        tris    PORTB           

        bsf     flags,ESTOPF    ; wake up with ESTOP on
        bsf     PORTB,ESTOP1
        bsf     PORTB,ESTOP2
        

        ;not needed?
        ;bsf    flags,LCF       ; wake up with lost carrier
        bsf     PORTB,LC



fstlow  btfss   PORTB,INPULSE   ; wait for rising edge on input
        goto    fstrise
        call    LocButton
        goto    fstlow

fstrise btfsc   PORTB,INPULSE   ; wait for falling edge on iput
        goto    LCResetWaitFall
        call    LocButton
        goto    fstrise

LCResetWaitFall
        movlw   PORTBIO         ; WAS BEING RESET SOMEHOW - BRUTE FORCE FIX
        tris    PORTB           
        movlw   LCAddHits
        movwf   LCHits
ResetWaitFall
        call    ResetHits
waitFall        
        clrf    msCount         ; reset the counter     
wfLoop  movlw   DefaultDelay1   ; Load time unit = 0.01 mS
        ;call   DelaySub        ; delay - count - check rise

        nop
        nop
        nop

        incfsz  msCount,SAME
        goto    wfSkip

        bsf     PORTB,LC        ; Timeout on msCount
        goto    fstlow

wfSkip  btfsc   PORTB,INPULSE   ; wait for falling edge on input
        goto    wfLoop

ChkLocalButtons
        call    LocButton

classify
BPStest
        movlw   BPSTHD          ; get the threshold
        subwf   msCount,W
        btfsc   STATUS,CARRY    ; measure for BPS region
        goto    gotest          ; next measurment
                        ; clear / set outputs
        bsf     PORTB,LC        ; set lost carrier
        goto    fstrise

gotest
        movlw   GOTHD           ; get the threshold
        subwf   msCount,W
        btfsc   STATUS,CARRY    ; measure for GO region
        goto    normtest        ; next measurment
                        ; clear / set flags   
        bsf     flags,GOF       ; report a go   
        bcf     flags,NORMF     
        bcf     flags,ESTOPF    
        goto    waitrise

normtest
        movlw   NORMTHD         ; get the threshold
        subwf   msCount,W
        btfsc   STATUS,CARRY    ; measure for NORM region
        goto    estptest        ; next measurment
                        ; clear / set flags   
        bcf     flags,GOF               
        bsf     flags,NORMF     ; report a norm
        bcf     flags,ESTOPF
        goto    waitrise


        
estptest
        movlw   ESTPTHD         ; get the threshold
        subwf   msCount,W
        btfsc   STATUS,CARRY    ; measure for ESTOP region
        goto    bpltest         ; next measurment

        bcf     flags,GOF               
        bcf     flags,NORMF
        bsf     flags,ESTOPF    ; report an ESTOP
        goto    waitrise


bpltest         ; if here than pulse=too long
        bsf     PORTB,LC        ; set lost carrier
        goto    fstrise


                        ; wait for pulse to fall again
waitrise
        rrf     msCount,SAME    ; Divide msCount by 8
        rrf     msCount,SAME 
        rrf     msCount,SAME 
        movlw   0x1F
        andwf   msCount,SAME
        btfsc   STATUS,CARRY    ; Round
        incf    msCount,SAME

wrLoop  movlw   DefaultDelay2   ; Load time unit = 0.08 mS
        call    DelaySub        ; delay - count - check rise

        nop
        nop
        nop

        incfsz  msCount,SAME
        goto    wrSkip

;       decf    msCount,SAME
        bsf     PORTB,LC
        goto    fstrise

wrSkip  btfss   PORTB,INPULSE   ; wait for rising edge on input
        goto    wrLoop


sfTest  movlw   SFRAME          ; test for short frame
        subwf   msCount,W
        btfsc   STATUS,CARRY    ; measure for short frame
        goto    lfTest          ; next measurment

        bsf     PORTB,LC        ; set lost carrier
        goto    LCResetWaitFall


lfTest  movlw   LFRAME          ; test for long frame
        subwf   msCount,W
        btfss   STATUS,CARRY    ; measure for long frame
        goto    okFrame         ; next measurment

        bsf     PORTB,LC        ; set lost carrier
        goto    LCResetWaitFall

okFrame bcf     PORTB,LC        ; clear lost carrier 
        clrf    LCHits          ; no extra hits needed if frame ok
        
goFtest
        btfss   flags,GOF
        goto    normFtest
        movlw   NumHits
        movwf   EstpCnt

        decfsz  GoCount,SAME
        goto    waitFall        

        bcf     PORTB,ESTOP1
        bcf     PORTB,ESTOP2
        bsf     PORTB,GO
        goto    ResetWaitFall

normFtest
        btfss   flags,NORMF
        goto    estopFtest

        bcf     PORTB,GO
        goto    ResetWaitFall

estopFtest
        movlw   NumHits
        movwf   GoCount
        decfsz  EstpCnt,SAME
        goto    waitFall

        bcf     PORTB,GO
        bsf     PORTB,ESTOP1
        bsf     PORTB,ESTOP2
        goto    ResetWaitFall

 ;********************************************************
 ;     Local Button test
 ;--------------------------------------------------------

LocButton
LESTOPtest
        btfsc   PORTB,LESTOP    ; check for local estop
        goto    LGOtest
        
ChkLESTOP_Again
        movlw   Debounce
        call    DelaySub

        btfsc   PORTB,LESTOP    ; check agian
        goto    LGOtest         ; leave with estop still set

        bcf     PORTB,GO        ; clear go pin  
        bsf     PORTB,ESTOP1    ; set estop pin
        bsf     PORTB,ESTOP2    ; set estop pin
        goto    ChkLESTOP_Again ; stay here til button released


LGOtest
        btfsc   PORTB,LGO       ; check for local go
        goto    DoneLocButton

ChkLGO_Again    
        movlw   Debounce
        call    DelaySub

        btfsc   PORTB,LGO       ; check agian
        goto    LGOoff

        bcf     PORTB,ESTOP1    ; clear the estop pin
        bcf     PORTB,ESTOP2    ; clear the estop pin
        bsf     PORTB,GO        ; set go pin
        goto    ChkLGO_Again    ; stay here till button released

LGOoff  bcf     PORTB,GO        ; clear go pin

DoneLocButton
        retlw   0


 ;************************************************************
 ;                      Reset Hits
 ;------------------------------------------------------------
ResetHits
        movf    LCHits,W        ; value=0 if frame is ok
        addlw   NumHits         ; hits required for action 
        movwf   GoCount
        movwf   EstpCnt
        retlw   0

        
 ;************************************************************
 ;                      Delay Subroutine
 ;------------------------------------------------------------
        
DelaySub                        ; delay for amount indicated in W
        movwf   delay1          
wait    decfsz  delay1,SAME     ; count down
        goto    wait
        retlw   0               ; done delay

        

        END






